use rand::distr::{Alphanumeric, SampleString};
use sqlx::{PgPool, Postgres, QueryBuilder};
use validator::Validate;

use crate::model::member::{Groups, SwimGroups};

#[derive(Debug, Validate, sqlx::FromRow)]
pub struct Member {
    #[validate(length(equal = 7))]
    pub member_id: String,
    pub first_name: String,
    pub full_name: String,
    pub registration_token: Option<String>,
    pub diploma: Option<String>,
    pub swim_groups: SwimGroups,
    pub groups: Groups,
}

impl Member {
    pub async fn get_many_from_registration_tokens(
        pool: &PgPool,
        registration_tokens: Vec<String>,
    ) -> Result<Vec<Self>, sqlx::Error> {
        let members = sqlx::query_as!(
            Member,
            "SELECT * FROM members WHERE registration_token = ANY($1);",
            &registration_tokens
        )
        .fetch_all(pool)
        .await?;

        Ok(members)
    }

    pub async fn get_many(transaction: &PgPool, members: Vec<Self>) -> Result<(), sqlx::Error> {
        Ok(())
    }

    pub async fn get_all(pool: &PgPool) -> Result<Vec<Self>, sqlx::Error> {
        let members = sqlx::query_as!(Member, "SELECT * FROM members;",)
            .fetch_all(pool)
            .await?;

        Ok(members)
    }

    pub async fn insert_many(
        transaction: &mut sqlx::Transaction<'_, Postgres>,
        members: Vec<Self>,
    ) -> Result<(), sqlx::Error> {
        if members.len() == 0 {
            return Ok(());
        }

        let mut query_builder = QueryBuilder::new(
            "INSERT INTO members(member_id, first_name, full_name, registration_token, diploma, swim_groups, groups) "
        );

        query_builder.push_values(members.into_iter(), |mut b, member| {
            let registration_token = Alphanumeric.sample_string(&mut rand::rng(), 16);

            b.push_bind(member.member_id);
            b.push_bind(member.first_name);
            b.push_bind(member.full_name);
            b.push_bind(registration_token);
            b.push_bind(member.diploma);
            b.push_bind(member.swim_groups.bits() as i64);
            b.push_bind(member.groups.bits() as i64);
        });

        let query = query_builder.build();
        query.execute(&mut **transaction).await?;

        Ok(())
    }

    pub async fn update_many(
        transaction: &mut sqlx::Transaction<'_, Postgres>,
        members: Vec<Self>,
    ) -> Result<(), sqlx::Error> {
        if members.len() == 0 {
            return Ok(());
        }

        for member in members {
            sqlx::query!(
                "
                    UPDATE ONLY members
                    SET first_name = $1, full_name = $2, diploma = $3, swim_groups = $4, groups = $5
                    WHERE member_id = $6
                ",
                member.first_name,
                member.full_name,
                member.diploma,
                member.swim_groups.bits() as i64,
                member.groups.bits() as i64,
                member.member_id
            )
            .execute(&mut **transaction)
            .await?;
        }

        Ok(())
    }

    pub async fn remove_many(
        transaction: &mut sqlx::Transaction<'_, Postgres>,
        member_ids: &Vec<String>,
    ) -> Result<(), sqlx::Error> {
        sqlx::query!(
            "
                DELETE FROM members WHERE member_id = ANY($1)
            ",
            member_ids
        )
        .execute(&mut **transaction)
        .await?;

        Ok(())
    }
}
